:root {
    --bs-bg-opacity: 1;
    --bg-color: #f3daa6;
    --bs-dark-rgb: 28, 36, 41 !important;
    --bs-bg-opacity: 1;
}

.table-dark {
    --bs-table-bg: 28, 36, 41 !important;
}

.btn-primary {
    --bs-btn-bg: #26566c;
    --bs-btn-border-color: #26566c;
}

body.theme-dark {
    background-image: linear-gradient(90deg, #1c2429, #263238);
    color: #CFD8DC;
}

.list-group .list-group-item {
    background: #343a40;
}

.theme-dark .navbar-nav i,
.navbar-nav .dropdown-toggle,
.break-word {
    color: #CFD8DC;
}

a,
a:hover,
a:visited,
a:active,
#main-table .filename a,
i.fa.fa-folder-o,
i.go-back {
    color: var(--bg-color);
}

ul#search-wrapper li:nth-child(odd) {
    background: #212a2f;
}

.theme-dark .btn-outline-primary {
    color: #b8e59c;
    border-color: #b8e59c;
}

.theme-dark .btn-outline-primary:hover,
.theme-dark .btn-outline-primary:active {
    background-color: #2d4121;
}

.theme-dark input.form-control {
    background-color: #101518;
    color: #CFD8DC;
}

.theme-dark .dropzone {
    background: transparent;
}

.theme-dark .inline-actions>a>i {
    background: #79755e;
}

.theme-dark .text-white {
    color: #CFD8DC !important;
}

.theme-dark .table-bordered td,
.table-bordered th {
    border-color: #343434;
}

.theme-dark .table-bordered td .custom-control-input,
.theme-dark .table-bordered th .custom-control-input {
    opacity: 0.678;
}

.message {
    background-color: #212529;
}

.compact-table tr:hover td {
    background-color: #3d3d3d;
}

#main-table tr.even {
    background-color: #21292f;
}

form.dropzone {
    border-color: #79755e;
}