.fm_custom_navbar {
    margin-bottom: 1.5rem;
    display: flex;
    align-items: center;
}

.fm_custom_navbar--right {
    width: 100%;
    display: flex;
    align-items: center;
    justify-content: flex-end;
    column-gap: 10px;
}

.fm_action_buttons {
    display: flex;
    align-items: center;
    margin-bottom: 1rem;
}

/**
 * REMOVE ONCE ON BOOTSTRAP 5
 */
.ms-2 {
    margin-left: 0.5rem;
}