<?php

namespace onespace\tools\assets;

use yii\web\AssetBundle;

/**
 * {@inheritdoc}
 * 
 * @author  Gareth Palmer   <gareth@one-space.co.za>
 * 
 * @deprecated  2024-08-23 - Moved to onespace\tools\icons\_rest_of_namespace_
 */

final class FontAwesome5Asset extends AssetBundle {
    public $sourcePath = '@vendor/onespace/yii2-widgets/src/web';

    public $css = [
        'css/fa5.min.css',
    ];

}