.datacard-padding-overwrite {
    padding: unset;
}

.datacard-data-container {
    display: grid;
    grid-template-columns: 1fr;
    grid-template-rows: 350px 1fr;
}

.datacard-image-container-placeholder {
    place-self: center;
}

.datacard-image-container {
    overflow: hidden;
}

.datacard-image {
    width: 100%;
    height: 100%;
    object-fit: cover;
}

.datacard-info-container {
    padding: var(--panel-padding);
    display: grid;
    grid-template-columns: 1fr;
    grid-template-rows: repeat(2, 1fr);
    grid-gap: 1rem;
}

.datacard-info-details {
    display: grid;
    grid-template-columns: 1fr;
    grid-template-rows: repeat(2, 1fr);
    font-size: 1.5rem;
    overflow-wrap: break-word;
    word-break: break-word;
}

.datacard-info-details-heading {
    margin: 1rem 0;
    font-weight: 600;
    color: var(--smokey-grey);
}

.datacard-info-details-value {
    font-weight: 600;
    color: #000;
    font-size: 2.5rem;
}

@media screen and (min-width: 768px) {
    .datacard-info-container {
        grid-template-columns: repeat(2, 1fr);
        grid-template-rows: auto;
    }

    .datacard-info-details {
        grid-template-columns: 40% 60%;
        grid-template-rows: 1fr;
    }

    .datacard-info-details-value {
        margin: 1rem 0;
        font-size: inherit;
    }
}