<?php

namespace onespace\tools\assets;

use yii\web\AssetBundle;
use yii\web\View;

/**
 * Asset bundle for tracking code
 * 
 * In order to enable Clarity tracking, the CLARITY_TRACKING_PROJECT environment variable should be set to the Clarity project ID
 */
class TrackingAsset extends AssetBundle {
    public $sourcePath = __DIR__;
    // public $basePath = '@webroot';
    // public $baseUrl = '@web';
    public $css = [];
    public $js = [
        [
            'js/clarity.js',
            'position' => View::POS_HEAD,
        ]
    ];
    public $depends = [];
}
