:root {
  --panel-border-radius: 6px;
  --panel-shadow: 0.6px 0.6px 0.8px rgba(0, 0, 0, 0.02),
    1.3px 1.3px 2px rgba(0, 0, 0, 0.028), 2.5px 2.5px 3.8px rgba(0, 0, 0, 0.035),
    4.5px 4.5px 6.7px rgba(0, 0, 0, 0.042),
    8.4px 8.4px 12.5px rgba(0, 0, 0, 0.05), 20px 20px 30px rgba(0, 0, 0, 0.07);
  --panel-min-height: 85vh;
  --panel-padding: 1.5rem;
  --panel-padding-mobile: 1rem;
}

@media screen and (min-width: 768px) {
  :root {
    --panel-padding: 2.5rem;
  }
}

.ui-rp-content-header {
  padding: unset;
}

.ui-rp-content-header .narrow-banner,
.ui-rp-content-header .ui-panel {
  border-radius: unset;
}

.ui-pane-container-left .narrow-banner,
.ui-pane-container-right .narrow-banner {
  border-top-left-radius: var(--panel-border-radius);
  border-top-right-radius: var(--panel-border-radius);
}

@media screen and (min-width: 768px) {
  .ui-rp-content-header {
    padding: 15px 15px 0 15px;
  }

  .ui-rp-content-header .ui-panel {
    border-radius: var(--panel-border-radius);
  }

  .ui-rp-content-header .narrow-banner {
    border-top-left-radius: var(--panel-border-radius);
    border-top-right-radius: var(--panel-border-radius);
  }
}

@media screen and (max-width: 768px) {
  .ui-page-content {
    padding: 0;
  }
}

.ui-page {
  min-height: var(--panel-min-height);
  display: grid;
  overflow-x: auto;
}

.ui-page:has(.portal-section) {
  min-height: 65vh;
}

.ui-page:has(.rp-sub-portal) {
  min-height: 68vh;
}

.ui-panel {
  border-radius: var(--panel-border-radius);
  box-shadow: var(--panel-shadow);
  padding: var(--panel-padding-mobile);
  background-color: #fff;
  color: var(--main-colour);
  align-self: stretch;
}

.ui-userinfo-container {
  background-color: var(--main-colour);
  color: #fff;
}

.ui-outline-whitespace {
  outline: 1px solid var(--whitespace);
}

.ui-panel-duel {
  display: grid;
  padding: var(--panel-padding-mobile);
  gap: 2.5rem;
  grid-template-rows: 1fr;
}

.ui-panel-duel-resizeable {
  gap: 1.3rem;
}

.ui-panel-0-template {
  grid-template-columns: repeat(2, 1fr);
}

.ui-panel-0r-template {
  grid-template-columns: 1fr 2px 1fr;
}

.ui-panel-1-template {
  grid-template-columns: 1fr 3fr;
}

.ui-panel-1r-template {
  grid-template-columns: 1fr 2px 3fr;
}

.ui-panel-2-template {
  grid-template-columns: 3fr 1fr;
}

.ui-panel-2r-template {
  grid-template-columns: 3fr 2px 1fr;
}

.ui-panel-3-template {
  grid-template-columns: 1fr 2fr;
}

.ui-panel-3r-template {
  grid-template-columns: 1fr 2px 2fr;
}

.ui-panel-4-template {
  grid-template-columns: 2fr 1fr;
}

.ui-panel-4r-template {
  grid-template-columns: 2fr 2px 1fr;
}

.ui-duel-pane-resizer {
  background-color: black;
  opacity: 5%;
  cursor: col-resize;
}

.ui-duel-pane-resizer:hover {
  opacity: 55%;
}

.ui-duel-pane-resizer:active {
  opacity: 75%;
}

.ui-panel-duel-resizeable .ui-pane-container {
  overflow: hidden;
}

@media screen and (max-width: 1395px) {
  .ui-panel-duel {
    grid-template-columns: 1fr;
    grid-template-rows: max-content 1fr;
  }

  .ui-panel-duel-resizeable {
    grid-template-columns: 1fr;
    grid-template-rows: max-content 0 1fr;
  }
}

@media screen and (min-width: 528px) {
  .ui-page {
    overflow: visible;
  }
}

@media screen and (min-width: 768px) {
  .ui-panel {
    padding: var(--panel-padding);
  }

  .ui-panel-duel {
    padding: 1.5rem;
  }
}

.ui-pane-container {
  min-height: inherit;
  background-color: #f1f1f1;
  padding: unset;
}

.ui-pane-content {
  padding: var(--panel-padding);
}

.item-header-container {
  display: grid;
  column-gap: 1.3rem;
  grid-template-columns: repeat(auto-fit, minmax(50px, 1fr));
  grid-template-rows: auto;
  align-items: center;
}

.ui-item-hr {
  margin-top: 0.7rem;
  margin-bottom: 0;
}

.item-header-left {
  justify-self: start;
}

.item-header-center {
  justify-self: center;
}

.item-header-right {
  justify-self: end;
}

.ui-item-header-text {
  color: var(--main-colour);
  font-size: 2rem;
  font-weight: 800;
}

@media screen and (min-width: 1048px) {
  .ui-item-header-text {
    font-size: 3rem;
  }
}

.ui-btn-action {
  background-color: var(--main-accent);
  color: #fff;
  font-weight: bold;
  padding: 6px 40px;
}

.ui-filter-button {
  color: var(--main-accent);
  font-size: 3rem;
  cursor: pointer;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.ui-table {
  border-collapse: separate;
  border-spacing: 0 1.2rem;
}

.ui-table-row {
  background-color: #fff;
  margin: 2em 0;
  outline: 1px solid var(--cool-grey);
  border-radius: var(--panel-border-radius);
  box-shadow: 2px 2px 5px -1px #7f7f7f;
}

.ui-table thead tr th,
.ui-table thead tr td {
  border: none;
}

.ui-table thead tr th,
.ui-table thead tr th a {
  color: var(--smokey-grey);
}

.ui-table thead tr th a:hover {
  color: var(--main-accent);
}

.ui-table thead tr th a.asc,
.ui-table thead tr th a.desc {
  color: var(--main-accent);
}

.ui-table-row:hover {
  outline-color: var(--main-colour);
}

.ui-table-row td:first-child {
  border-radius: var(--panel-border-radius) 0 0 var(--panel-border-radius);
}

.ui-table-row td:last-child {
  border-radius: 0 var(--panel-border-radius) var(--panel-border-radius) 0;
}

.ui-outline-main {
  outline: 2px solid var(--main-colour);
}

.ui-outline-accent {
  outline: 2px solid var(--main-accent);
}

.ui-outline-smokeygrey {
  outline: 2px solid var(--smokey-grey);
}

.ui-outline-coolgrey {
  outline: 2px solid var(--cool-grey);
}

.ui-outline-snipr {
  outline: 2px solid var(--snipr-orange);
}

.ui-unset-top-corners {
  border-top-left-radius: unset;
  border-top-right-radius: unset;
}

.ui-unset-bottom-corners {
  border-bottom-left-radius: unset;
  border-bottom-right-radius: unset;
}

.ui-unset-shadow {
  box-shadow: unset;
}

.ui-data-cards-view {
  display: grid;
  grid-template-columns: 1fr;
  grid-template-rows: auto;
  gap: 2.5rem;
  margin-top: 1.5rem;
}

.ui-card-container {
  font-size: 1.6rem;
  border-radius: var(--panel-border-radius);
  max-width: 85vw;
  display: grid;
}

.ui-card-container a {
  align-self: center;
}

.ui-card-container[data-selected] {
  border: 2px solid var(--main-accent);
  background-color: #fff;
}

.ui-card-container:hover {
  transform: scale(1.02);
  box-shadow: var(--panel-shadow);
}

.ui-card-image {
  grid-area: image;
  align-self: center;
  border-radius: 50%;
  height: 100px;
  border: 2px solid var(--main-colour);
  overflow: hidden;
  display: flex;
  justify-content: center;
  align-items: center;
  text-align: center;
}

.ui-card-container[data-selected] .ui-card-image {
  border-color: var(--main-accent);
}

.ui-card-heading {
  grid-area: heading;
  font-weight: 700;
  color: var(--main-colour);
  font-size: 2.4rem;
  align-self: center;
}

.ui-card-heading b {
  overflow-wrap: break-word;
  word-break: break-word;
}

.ui-card-details {
  grid-area: details;
  color: var(--smokey-grey);
  font-weight: 500;
  display: none;
}

.ui-card-buttons {
  justify-self: end;
  grid-area: buttons;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
}

.ui-card-edit {
  color: var(--main-accent);
  grid-area: edit-button;
}

.ui-card-delete {
  color: var(--main-accent);
  grid-area: delete-button;
}

.ui-data-card-context-button {
  font-size: 2.3rem;
}

.ui-data-card-context-button:hover,
.ui-header-icon-button:hover {
  transform: scale(1.1);
}

.ui-card-elements {
  padding: 1.3rem;
  display: grid;
  grid-template-columns: 100px 1fr max-content;
  grid-template-rows: auto;
  column-gap: 1.3rem;
  grid-template-areas:
    "image . edit-button"
    "image heading heading"
    "image . delete-button";
}

@media screen and (min-width: 841px) {
  .ui-data-cards-view {
    grid-template-columns: 1fr 1fr;
  }

  .ui-card-elements {
    grid-template-columns: 100px repeat(2, 13px) 1fr;
    grid-template-areas:
      "image heading heading heading"
      "image details details details"
      "image edit-button delete-button .";
  }

  .ui-card-buttons {
    justify-self: unset;
    flex-direction: row;
    justify-content: unset;
  }

  .ui-card-heading {
    font-size: unset;
    align-self: unset;
  }

  .ui-card-details {
    display: block;
  }

  .ui-data-card-context-button {
    font-size: 1.7rem;
  }
}

@media screen and (min-width: 920px) {
  .ui-card-image {
    height: 140px;
  }

  .ui-card-elements {
    grid-template-columns: 140px repeat(2, 13px) 1fr;
  }
}

.ui-pet-placeholder {
  font-size: 5rem;
  color: var(--cool-grey);
  width: 50px;
  height: 50px;
}

.ui-calendar-background {
  background-color: #fff;
}

.ui-calendar-br {
  border-radius: 3px;
}

.ui-calendar-details-padding {
  padding: var(--panel-padding);
}

.ui-selected-row {
  outline: 2px solid var(--main-accent);
}

.ui-slider-container {
  display: flex;
  align-items: center;
  column-gap: 6px;
}

.ui-main-label {
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.ui-switch {
  position: relative;
  display: inline-block;
  width: 50px;
  height: 28px;
}

.ui-switch [data-type="ui-switcher"] {
  opacity: 0;
  width: 0;
  height: 0;
}

.ui-slider {
  position: absolute;
  cursor: pointer;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  background-color: #ccc;
  -webkit-transition: 0.4s;
  transition: 0.4s;
}

.ui-slider:before {
  position: absolute;
  content: "";
  height: 20px;
  width: 20px;
  left: 4px;
  bottom: 4px;
  background-color: white;
  -webkit-transition: 0.4s;
  transition: 0.4s;
}

[data-type="ui-switcher"]:checked + .ui-slider {
  background-color: var(--main-colour);
}

[data-type="ui-switcher"]:focus + .ui-slider {
  box-shadow: 0 0 1px var(--main-colour);
}

[data-type="ui-switcher"]:checked + .ui-slider:before {
  -webkit-transform: translateX(21px);
  -ms-transform: translateX(21px);
  transform: translateX(21px);
}

.ui-slider.ui-slider-round {
  border-radius: 34px;
}

.ui-slider.ui-slider-round:before {
  border-radius: 50%;
}
