<?php

namespace onespace\tools\widgets\ui\banner\base;

/**
 * Required methods to topbanner widget classes.
 * 
 * @author  Gareth Palmer <gareth@one-space.co.za>
 */

interface BannerTemplate {

    /**
     * {@inheritdoc}
     * 
     * @return  string
     * 
     * @access  public
     */

    public function run(): string;


    /**
     * Contains css specific to the desired banner.
     * 
     * @return  string
     * 
     * @access  public
     */

    public function banner_css(): string;
}