<?php

namespace onespace\tools\helpers\traits;

/** @deprecated for CLIHelper */
trait TerminalFeedback {

    protected function displayProgress($completeCount, $totalCount) {
        $spinnerStates = ['|', '\\', '-', '/'];
        $spinnerChar = $spinnerStates[$completeCount % 4];
        echo "\r Processing: " . $completeCount . " / $totalCount complete (" . (($completeCount / $totalCount) * 100) . "%) [$spinnerChar]";
    }
}
