<?php

namespace onespace\tools\assets;

use yii\web\AssetBundle;

class OSWidgetsAssets extends AssetBundle {
    public $sourcePath = '@vendor/onespace/yii2-widgets/src/widgets/ui/assets';

    public $css = [
        'css/colours.min.css',
        'css/ui.min.css',
    ];

    public $js = [
        'js/ui-onespace.min.js',
    ];
}
