<?php

namespace onespace\tools\widgets\loaders;

final class SpinnerLoader extends \onespace\tools\widgets\loaders\BaseIconLoader {

    /**
     * {@inheritdoc}
     */

    public function init() {
        parent::init();
        $this->widget['identifier'] = 'loader';
    }
}
