<?php

namespace onespace\tools\widgets\loaders;

final class LineSpinLoader extends \onespace\tools\widgets\loaders\BaseIconLoader {

    public bool $drawPath = false;

    /**
     * {@inheritdoc}
     */

    public function init() {
        parent::init();
        $this->widget['identifier'] = 'spinner-third';
        $this->widget['style'] = $this->drawPath ? 'duotone' : null;
    }
}
