<?php

namespace onespace\tools\widgets\icons;

use onespace\tools\widgets\icons\base\Icon;

/**
 * Generate Gylphicon library icons as a widget.
 * 
 * @property    string|null $identifier See `https://getbootstrap.com/docs/3.3/components/#glyphicons`
 * @property    string|null $colour
 * @property    string  $content    Default: `&nbsp;`
 * 
 * @author  Jason Phillips <jason@one-space.co.za>
 * 
 * @deprecated  2024-08-23 - Moved to onespace\tools\icons\_rest_of_namespace_
 */

final class Glyphicon extends Icon {
    protected string $baseClass = 'glyphicon glyphicon-';
    protected string $defaultIdentifier = 'question-sign';
}
