<?php

namespace onespace\tools\assets;

use yii\web\View;

class LocationPickerAsset extends \yii\web\AssetBundle {
    public $sourcePath = '@vendor/projector22/jquery-location-picker/dist';
    public $css = [];
    public $js = [
        'locationpicker.jquery.min.js',
    ];
    public $jsOptions = [
        'position' => View::POS_END,
    ];

    public $depends = [
        'yii\web\JqueryAsset',
    ];
}
