<?php

namespace onespace\tools\widgets\icons\base;

/**
 * @deprecated  2024-08-23 - Moved to onespace\tools\icons\_rest_of_namespace_
 */
trait FAHelper {
    public ?string $style = null;

    public function init() {
        $this->baseClass = ' fa-';
        $this->defaultIdentifier = 'question';

        if ($this->style == null || !isset(self::class::STYLES[$this->style])) {
            $this->style = 'solid';
        }
        $this->baseClass = self::class::STYLES[$this->style] . $this->baseClass;

        parent::init();
    }
}
