<?php

@symlink('/var/www/html/vendor/onespace/yii2-widgets/src/ui/assets/css/colours.min.css', '/var/www/html/web/css/colours.min.css');
@symlink('/var/www/html/vendor/onespace/yii2-widgets/src/ui/assets/css/ui.min.css', '/var/www/html/web/css/ui.min.css');
@symlink('/var/www/html/vendor/onespace/yii2-widgets/src/ui/assets/js/ui-onespace.min.js', '/var/www/html/web/js/ui-onespace.min.js');

echo "Onespace Widgets Post Install complete... \n";