.general-narrow-banner {
    display: grid;
    padding: 1rem 3.5rem;
    align-items: center;
    grid-template-columns: 140px 1fr;
    grid-template-rows: auto;
    grid-template-areas:
        "profile-pic user-text";
}

.profile-pic {
    grid-area: profile-pic;
    justify-self: start;
}

.profile-pic-img {
    border: 3px solid var(--main-accent);
    border-radius: 50%;
    height: 120px;
    width: 120px;
    object-fit: cover;
}

.profile-info {
    grid-area: user-text;
    color: var(--cool-grey);
}

.profile-info .h3 {
    color: var(--main-accent);
    margin-top: 0;
    margin-bottom: 0;
}

.profile-info .h3 {
    color: var(--main-accent);
    margin-top: 0;
    margin-bottom: 0;
}

.profile-link {
    color: var(--cool-grey);
    font-size: 1.8rem;
}

.profile-link:hover {
    color: #fff;
}

.profile-info-sub div {
    margin-top: 3px;
}