.narrow-banner {
    height: max-content;
    min-height: 70px;
}

.title-banner {
    padding: 0.5rem;
    display: grid;
    grid-template-columns: max-content 1fr;
    align-items: center;
    padding-left: 3.1rem;
    padding-right: 3.1rem;
}

.ui-text-header {
    font-size: 2rem;
    font-weight: 300;
}

@media screen and (min-width: 768px) {
    .ui-text-header {
        font-size: 26px;
    }
}

.ui-text-links {
    color: var(--main-accent);
    font-size: 3.0rem;
}

.ui-text-links span {
    margin-left: 0.5rem;
}

.ui-text-links span a {
    color: var(--main-accent);
    text-decoration: none;
}