<?php

namespace onespace\tools\helpers;

/**
 * @todo    replace with ENUM as soon as everyone is running > PHP 8.1
 */

class Position {
    CONST LEFT = 'l';
    CONST CENTER = 'c';
    CONST RIGHT = 'r';

    CONST TOP = 't';
    CONST MIDDLE = 'm';
    CONST BOTTOM = 'b';
}

/*
enum Position: string {
    case LEFT = 'l';
    case MIDDLE = 'm';
    case RIGHT = 'r';

    case TOP = 't';
    case MIDDLE = 'm';
    case BOTTOM = 'b';
}
*/