<?php

namespace onespace\tools\assets;

/**
 * {@inheritdoc}
 */

class ChartJSAsset extends \yii\web\AssetBundle {

    /**
     * {@inheritdoc}
     */
    public $js = [
        'https://cdn.jsdelivr.net/npm/chart.js',
    ];
}
