window.startMap = () => {
    /**
     * This function does nothing, but needs to exist to prevent an error.
     * 
     * @since 2023-01-19, the Google Maps API requires a callback function. As this API is called
     * on every page load regardless, a function must be defined to prevent a random error. This
     * is that function.
     */
    if (typeof mapCallback === 'function') {
        mapCallback();
    }
};