.profile-pic-img__fallback {
    color: var(--cool-grey);
    display: grid;
    place-items: center;
    overflow: hidden;
    position: relative;
}

.profile-pic-img__fallback-svg {
    width: 100%;
    height: 100%;
    z-index: 50;
    position: absolute;
}

.profile-pic-img__fallback-svg>text {
    fill: var(--cool-grey);
    font-size: 2.0rem;
    letter-spacing: .1rem;
    stroke: black;
    stroke-width: 0.3rem;
    stroke-linejoin: round;
    paint-order: stroke;
    text-shadow: #363636 1px 0 3px;
}